#include "predef.h"
#include <stdio.h>
#include <constants.h>
#include <basictypes.h>
#include <utils.h>
#include <iosys.h>
#include <dns.h>
#include <ip.h>
#include <tcp.h>
#include <base64.h>
#include <string.h>

#define	DATA_BUFSIZE	50000
#define	RX_BUFSIZE		8192

extern "C" {
void getTwitterStatus(int sock, PCSTR url);
void postTwitterUpdate(int sock, PCSTR url);
}

char RxBuffer[RX_BUFSIZE];
char DataBuffer[DATA_BUFSIZE];


#error You must add username and password for your twitter account here.
char un[] = ""; // Twitter user name
char pw[] = ""; // Twitter password


// Function to execute a DNS lookup to obtain the resultant IP address
IPADDR GetIPAddress(char *name) {
	IPADDR ip_addr = 0;
	iprintf("Resolving URL: \"%s\"...\r\n", name);
	int status = GetHostByName(name, &ip_addr, 0, TICKS_PER_SECOND * 10);
	switch (status) {
	case DNS_OK:
		iprintf("URL: \"%s\" resolved to ", name);
		ShowIP(ip_addr);
		iprintf("\r\n");
		break;
	case DNS_TIMEOUT:
		iprintf("*** DNS ERROR: TIMEOUT!\r\n");
		break;
	case DNS_NOSUCHNAME:
		iprintf("*** DNS ERROR: NO SUCH NAME!\r\n");
		break;
	case DNS_ERR:
		iprintf("*** DNS ERROR: RESPONSE ERROR!\r\n");
		break;
	}
	return ip_addr;
}

// Function to get the latest Twitter status for a user
void getTwitterStatus(int sock, PCSTR url) {
	char method[] = "GET";
	char host[] = "api.twitter.com";
	char hosturl[] = "/1/users/show.json?id=NetBurner";
	char useragent[] = "NetBurner/PK70";

	// Getting IP Address
	IPADDR DestIP_Addr;
	while ((DestIP_Addr = GetIPAddress(host)) == 0) {
		OSTimeDly(TICKS_PER_SECOND * 5);
	}

	// Building GET Command
	char cmd[1024];
	int sizeOfMethod = 0;
	sizeOfMethod = siprintf(cmd, 	"%s %s HTTP/1.1\r\n"
									"Host: %s\r\n"
									"User-Agent: %s\r\n"
									"\r\n",
									method, hosturl, host, useragent);

	// Sending GET
	int fd = connect(DestIP_Addr, 0, 80, 5 * TICKS_PER_SECOND);
	write(fd, cmd, sizeOfMethod);

	// Reading Results
	// Loop and record all incoming data
	DWORD BytesRead = 0;
	DataBuffer[0] = '\0';
	while ((fd > 0) && (BytesRead < DATA_BUFSIZE - 1)) {
		int n = ReadWithTimeout(fd, RxBuffer, RX_BUFSIZE, TICKS_PER_SECOND * 5);
		if (n > 0) {
			BytesRead += n;
			RxBuffer[n] = '\0';
			//iprintf("Read %d bytes: %s\n", n, RxBuffer);
			iprintf("BytesRead: %d\r", BytesRead);

			if ((BytesRead + n) < DATA_BUFSIZE)
				strncat(DataBuffer, RxBuffer, n);
		} else // connection closed by server or timeout
		{
			iprintf("Connection closed by server, n = %d\r\n", n);
			close(fd);
			fd = 0;
		}
	}
	DataBuffer[BytesRead] = '\0';

	// Parsing Results
	int parseState = 0;
	char *cp = DataBuffer;
	while (*cp) {
		switch (parseState) {
		case 0:
			if (*cp == '"') {
				parseState = 1;
			} else
				parseState = 0;
			break;
		case 1:
			if (*cp == 't') {
				parseState = 20;
			} else {
				parseState = 0;
			}
			break;
		case 20:
			if (*cp == 'e') {
				parseState = 21;
			} else {
				parseState = 0;
			}
			break;
		case 21:
			if (*cp == 'x') {
				parseState = 22;
			} else {
				parseState = 0;
			}
			break;
		case 22:
			if (*cp == 't') {
				parseState = 23;
			} else {
				parseState = 0;
			}
			break;
		case 23:
			if (*cp == '"') {
				parseState = 24;
			} else {
				parseState = 0;
			}
			break;
		case 24:
			if (*cp == ':') {
				parseState = 25;
				iprintf("text: ");
			} else {
				parseState = 0;
			}
			break;
		case 25: // Got "text":
			if (*cp == '}') {
				parseState = 0;
				iprintf("\r\n");
			} else {
				iprintf("%c", *cp);
			}
			break;
		default:
			parseState = 0;
		}
		*cp++;
	}
	// Closing Socket
	close(fd);
}

// Function to post a twitter update
void postTwitterUpdate(int sock, PCSTR url) {
	char method[] = "POST";
	char host[] = "api.twitter.com";
	char hosturl[] = "/1/statuses/update.json";
	char useragent[] = "NetBurner";
	char update[] = "status=hello+world\r\n";

	// Generate username/password string
	char unpw[64];
	char p64Buff[64];
	Base64Encode((PBYTE) unpw, siprintf(unpw, "%s:%s", un, pw), p64Buff);

	// Getting IP Address
	IPADDR DestIP_Addr;
	while ((DestIP_Addr = GetIPAddress(host)) == 0) {
		OSTimeDly(TICKS_PER_SECOND * 5);
	}

	// Building POST Command
	char cmd[1024];
	int sizeOfMethod = 0;
	sizeOfMethod
			= siprintf(cmd, 	"%s %s HTTP/1.1\r\n"
								"Host: %s\r\n"
								"User-Agent: %s\r\n"
								"Content-Length: %d\r\n"
								"Authorization: Basic %s\r\n"
								"\r\n"
								"%s",
								method, hosturl, host, useragent,
								sizeof(update),	p64Buff, update);

	// Sending POST
	int fd = connect(DestIP_Addr, 0, 80, 5 * TICKS_PER_SECOND);
	write(fd, cmd, sizeOfMethod);

	// Closing Socket
	close(fd);
}

